@@ -5,7 +5,6 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import cPickle as pickle
 import os
 import sys
 import threading
@@ -19,6 +18,7 @@ from quodlibet import formats
 from quodlibet import qltk
 from quodlibet import util
 
+from quodlibet.compat import pickle
 from quodlibet.browsers import Browser
 from quodlibet.formats import AudioFile
 from quodlibet.formats.remote import RemoteFile
@@ -258,21 +258,30 @@ class AudioFeeds(Browser):
     @classmethod
     def write(klass):
         feeds = [row[0] for row in klass.__feeds]
-        f = file(FEEDS, "wb")
-        pickle.dump(feeds, f, pickle.HIGHEST_PROTOCOL)
-        f.close()
+        with open(FEEDS, "wb") as f:
+            pickle.dump(feeds, f, pickle.HIGHEST_PROTOCOL)
 
     @classmethod
     def init(klass, library):
+        uris = set()
         try:
-            feeds = pickle.load(file(FEEDS, "rb"))
+            with open(FEEDS, "rb") as fileobj:
+                feeds = pickle.load(fileobj)
         except (pickle.PickleError, EnvironmentError, EOFError):
             pass
         else:
             for feed in feeds:
+                if feed.uri in uris:
+                    continue
                 klass.__feeds.append(row=[feed])
+                uris.add(feed.uri)
         GLib.idle_add(klass.__do_check)
 
+    @classmethod
+    def reload(klass, library):
+        klass.__feeds = Gtk.ListStore(object)  # unread
+        klass.init(library)
+
     @classmethod
     def __do_check(klass):
         thread = threading.Thread(target=klass.__check, args=())
