@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
-#    2012 - 2015 Nick Boultbee
+#    2012 - 2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,36 +9,40 @@
 import urllib
 from gi.repository import Gtk, GLib, Pango, Gdk
 from tempfile import NamedTemporaryFile
-from quodlibet.plugins.playlist import PLAYLIST_HANDLER
 
-from quodlibet import config
+from quodlibet import config, print_d
 from quodlibet.browsers import Browser
+from quodlibet.browsers._base import DisplayPatternMixin
+from quodlibet.browsers.playlists.prefs import Preferences, \
+    DEFAULT_PATTERN_TEXT
 from quodlibet.formats import AudioFile
+from quodlibet.plugins.playlist import PLAYLIST_HANDLER
 from quodlibet.qltk.completion import LibraryTagCompletion
+from quodlibet.qltk.menubutton import MenuButton
+from quodlibet.qltk.models import ObjectStore, ObjectModelSort
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.songlist import SongList
+from quodlibet.qltk.songsmenu import SongsMenu
+from quodlibet.qltk.views import RCMHintedTreeView
+from quodlibet.qltk.x import ScrolledWindow, Align, MenuItem, SymbolicIconImage
 from quodlibet.query import Query
 from quodlibet.util import connect_obj
 from quodlibet.util.path import get_home_dir
-from quodlibet.qltk.songsmenu import SongsMenu
-from quodlibet.qltk.views import RCMHintedTreeView
-from quodlibet.qltk.models import ObjectStore, ObjectModelSort
-from quodlibet.qltk.x import ScrolledWindow, Align, MenuItem
-from quodlibet.qltk import Icons
-
 from .util import *
 
-
 DND_QL, DND_URI_LIST, DND_MOZ_URL = range(3)
 
 
-class PlaylistsBrowser(Browser):
+class PlaylistsBrowser(Browser, DisplayPatternMixin):
 
     name = _("Playlists")
     accelerated_name = _("_Playlists")
     keys = ["Playlists", "PlaylistsBrowser"]
     priority = 2
     replaygain_profiles = ["track"]
+    __last_render = None
+    _PATTERN_FN = os.path.join(quodlibet.get_user_dir(), "playlist_pattern")
+    _DEFAULT_PATTERN_TEXT = DEFAULT_PATTERN_TEXT
 
     def pack(self, songpane):
         self._main_box.pack1(self, True, False)
@@ -61,10 +65,11 @@ class PlaylistsBrowser(Browser):
         model = klass.__lists.get_model()
         for playlist in os.listdir(PLAYLISTS):
             try:
-                playlist = Playlist(PLAYLISTS, Playlist.unquote(playlist),
-                                    library=library)
+                playlist = FileBackedPlaylist(PLAYLISTS,
+                      FileBackedPlaylist.unquote(playlist), library=library)
                 model.append(row=[playlist])
             except EnvironmentError:
+                print_w("Invalid Playlist '%s'" % playlist)
                 pass
 
         klass._ids = [
@@ -72,6 +77,7 @@ class PlaylistsBrowser(Browser):
             library.connect('added', klass.__added),
             library.connect('changed', klass.__changed),
         ]
+        klass.load_pattern()
 
     @classmethod
     def deinit(cls, library):
@@ -120,30 +126,34 @@ class PlaylistsBrowser(Browser):
                     PlaylistsBrowser.changed(playlist, refresh=False)
                     break
 
-    @staticmethod
-    def cell_data(col, render, model, iter, data):
-        render.markup = model[iter][0].format()
-        render.set_property('markup', render.markup)
+    def cell_data(self, col, cell, model, iter, data):
+        playlist = model[iter][0]
+        cell.markup = markup = self.display_pattern % playlist
+        if self.__last_render == markup:
+            return
+        self.__last_render = markup
+        cell.markup = markup
+        cell.set_property('markup', markup)
 
     def Menu(self, songs, library, items):
-        songlist = qltk.get_top_parent(self).songlist
-        model, iters = self.__get_selected_songs(songlist)
-        item = qltk.MenuItem(_("_Remove from Playlist"), Icons.LIST_REMOVE)
-        qltk.add_fake_accel(item, "Delete")
-        connect_obj(item, 'activate', self.__remove, iters, model)
-        item.set_sensitive(bool(self.__view.get_selection().get_selected()[1]))
-
-        items.append([item])
+        model, iters = self.__get_selected_songs()
+        remove = qltk.MenuItem(_("_Remove from Playlist"), Icons.LIST_REMOVE)
+        qltk.add_fake_accel(remove, "Delete")
+        connect_obj(remove, 'activate', self.__remove, iters, model)
+        playlist_iter = self.__view.get_selection().get_selected()[1]
+        remove.set_sensitive(bool(playlist_iter))
+        items.append([remove])
         menu = super(PlaylistsBrowser, self).Menu(songs, library, items)
         return menu
 
-    def __get_selected_songs(self, songlist):
+    def __get_selected_songs(self):
+        songlist = qltk.get_top_parent(self).songlist
         model, rows = songlist.get_selection().get_selected_rows()
         iters = map(model.get_iter, rows)
         return model, iters
 
     __lists = ObjectModelSort(model=ObjectStore())
-    __lists.set_default_sort_func(lambda m, a, b, data: cmp(m[a][0], m[b][0]))
+    __lists.set_default_sort_func(ObjectStore._sort_on_value)
 
     def __init__(self, library):
         super(PlaylistsBrowser, self).__init__(spacing=6)
@@ -187,15 +197,19 @@ class PlaylistsBrowser(Browser):
         self.pack_start(swin, True, True, 0)
 
     def __configure_buttons(self, library):
-        newpl = qltk.Button(_("_New"), Icons.DOCUMENT_NEW, Gtk.IconSize.MENU)
-        newpl.connect('clicked', self.__new_playlist)
-        importpl = qltk.Button(_("_Import"), Icons.LIST_ADD, Gtk.IconSize.MENU)
-        importpl.connect('clicked', self.__import, library)
+        new_pl = qltk.Button(_("_New"), Icons.DOCUMENT_NEW, Gtk.IconSize.MENU)
+        new_pl.connect('clicked', self.__new_playlist)
+        import_pl = qltk.Button(_("_Import"), Icons.LIST_ADD,
+                                Gtk.IconSize.MENU)
+        import_pl.connect('clicked', self.__import, library)
         hb = Gtk.HBox(spacing=6)
-        hb.set_homogeneous(True)
-        hb.pack_start(newpl, True, True, 0)
-        hb.pack_start(importpl, True, True, 0)
-        self.pack_start(Align(hb, left=3, bottom=3), False, True, 0)
+        hb.set_homogeneous(False)
+        hb.pack_start(new_pl, True, True, 0)
+        hb.pack_start(import_pl, True, True, 0)
+        hb2 = Gtk.HBox(spacing=0)
+        hb2.pack_start(hb, True, True, 0)
+        hb2.pack_start(PreferencesButton(self), False, False, 6)
+        self.pack_start(Align(hb2, left=3, bottom=3), False, False, 0)
 
     def __create_playlists_view(self, render):
         view = RCMHintedTreeView()
@@ -205,7 +219,7 @@ class PlaylistsBrowser(Browser):
             lambda model, col, key, iter, data:
             not model[iter][col].name.lower().startswith(key.lower()), None)
         col = Gtk.TreeViewColumn("Playlists", render)
-        col.set_cell_data_func(render, PlaylistsBrowser.cell_data)
+        col.set_cell_data_func(render, self.cell_data)
         view.append_column(col)
         view.set_model(self.__lists)
         view.set_rules_hint(True)
@@ -238,6 +252,7 @@ class PlaylistsBrowser(Browser):
 
     def __create_cell_renderer(self):
         render = Gtk.CellRendererText()
+        render.set_padding(3, 3)
         render.set_property('ellipsize', Pango.EllipsizeMode.END)
         render.connect('editing-started', self.__start_editing)
         render.connect('edited', self.__edited)
@@ -250,8 +265,7 @@ class PlaylistsBrowser(Browser):
         return False
 
     def __handle_songlist_delete(self, *args):
-        songlist = qltk.get_top_parent(self).songlist
-        model, iters = self.__get_selected_songs(songlist)
+        model, iters = self.__get_selected_songs()
         self.__remove(iters, model)
 
     def __key_pressed(self, widget, event):
@@ -270,10 +284,7 @@ class PlaylistsBrowser(Browser):
         elif qltk.is_accel(event, "F2"):
             model, iter = self.__view.get_selection().get_selected()
             if iter:
-                self.__render.set_property('editable', True)
-                self.__view.set_cursor(model.get_path(iter),
-                                       self.__view.get_columns()[0],
-                                       start_editing=True)
+                self._start_rename(model.get_path(iter))
             return True
         return False
 
@@ -297,18 +308,35 @@ class PlaylistsBrowser(Browser):
         view.get_parent().drag_unhighlight()
 
     def __remove(self, iters, smodel):
+        def song_at(itr):
+            return smodel[smodel.get_path(itr)][0]
+
+        def remove_from_model(iters, smodel):
+            for it in iters:
+                smodel.remove(it)
+
         model, iter = self.__view.get_selection().get_selected()
         if iter:
-            for iter_remove in iters:
-                smodel.remove(iter_remove)
             playlist = model[iter][0]
-            # Calling playlist.remove_songs(songs) won't remove the right ones
-            # if there are duplicates
-            playlist.clear()
-            playlist.extend([row[0] for row in smodel])
+            if self._query is None or not self.get_filter_text():
+                # Calling playlist.remove_songs(songs) won't remove the
+                # right ones if there are duplicates
+                remove_from_model(iters, smodel)
+                self.__rebuild_playlist_from_songs_model(playlist, smodel)
+            else:
+                removals = [song_at(iter_remove) for iter_remove in iters]
+                print_d("Removing %d song(s) from %s"
+                        % (len(removals), playlist))
+                playlist.remove_songs(removals, True)
+                remove_from_model(iters, smodel)
             PlaylistsBrowser.changed(playlist)
             self.activate()
 
+    @classmethod
+    def __rebuild_playlist_from_songs_model(cls, playlist, smodel):
+        playlist.clear()
+        playlist.extend([row[0] for row in smodel])
+
     def __drag_data_received(self, view, ctx, x, y, sel, tid, etime, library):
         # TreeModelSort doesn't support GtkTreeDragDestDrop.
         view.emit_stop_by_name('drag-data-received')
@@ -322,7 +350,8 @@ class PlaylistsBrowser(Browser):
             try:
                 path, pos = view.get_dest_row_at_pos(x, y)
             except TypeError:
-                playlist = Playlist.fromsongs(PLAYLISTS, songs, library)
+                playlist = FileBackedPlaylist.from_songs(PLAYLISTS, songs,
+                                                         library)
                 GLib.idle_add(self._select_playlist, playlist)
             else:
                 playlist = model[path][0]
@@ -375,12 +404,15 @@ class PlaylistsBrowser(Browser):
         else:
             sel.set_uris([song("~uri") for song in songs])
 
-    def _select_playlist(self, playlist):
+    def _select_playlist(self, playlist, scroll=False):
         view = self.__view
         model = view.get_model()
         for row in model:
             if row[0] is playlist:
                 view.get_selection().select_iter(row.iter)
+                if scroll:
+                    view.scroll_to_cell(row.path, use_align=True,
+                                        row_align=0.5)
 
     def __popup_menu(self, view, library):
         model, itr = view.get_selection().get_selected()
@@ -406,8 +438,7 @@ class PlaylistsBrowser(Browser):
         menu.prepend(rem)
 
         def _rename(path):
-            self.__render.set_property('editable', True)
-            view.set_cursor(path, view.get_columns()[0], start_editing=True)
+            self._start_rename(path)
 
         ren = qltk.MenuItem(_("_Rename"), Icons.EDIT)
         qltk.add_fake_accel(ren, "F2")
@@ -419,28 +450,40 @@ class PlaylistsBrowser(Browser):
         menu.show_all()
         return view.popup_menu(menu, 0, Gtk.get_current_event_time())
 
+    def _start_rename(self, path):
+        view = self.__view
+        self.__render.set_property('editable', True)
+        view.set_cursor(path, view.get_columns()[0], start_editing=True)
+
     def __focus(self, widget, *args):
         qltk.get_top_parent(widget).songlist.grab_focus()
 
     def __text_parse(self, bar, text):
         self.activate()
 
-    def _get_text(self):
-        return self._sb_box.get_text()
-
     def _set_text(self, text):
         self._sb_box.set_text(text)
 
     def activate(self, widget=None, resort=True):
         songs = self._get_playlist_songs()
 
-        text = self._get_text()
+        text = self.get_filter_text()
         # TODO: remove static dependency on Query
         if Query.is_parsable(text):
             self._query = Query(text, SongList.star)
             songs = self._query.filter(songs)
         GLib.idle_add(self.songs_selected, songs, resort)
 
+    @classmethod
+    def refresh_all(cls):
+        model = cls.__lists.get_model()
+        for iter_, value in model.iterrows():
+            model.row_changed(model.get_path(iter_), iter_)
+
+    @property
+    def model(self):
+        return self.__lists.get_model()
+
     def _get_playlist_songs(self):
         model, iter = self.__view.get_selection().get_selected()
         songs = iter and list(model[iter][0]) or []
@@ -455,7 +498,7 @@ class PlaylistsBrowser(Browser):
         self.activate()
 
     def get_filter_text(self):
-        return self._get_text()
+        return self._sb_box.get_text()
 
     def can_filter(self, key):
         # TODO: special-case the ~playlists tag maybe?
@@ -475,26 +518,38 @@ class PlaylistsBrowser(Browser):
         model, iter = self.__view.get_selection().get_selected()
         name = iter and model[iter][0].name or ""
         config.set("browsers", "playlist", name)
-        text = self._get_text()
+        text = self.get_filter_text()
         config.set("browsers", "query_text", text)
 
     def __new_playlist(self, activator):
-        playlist = Playlist.new(PLAYLISTS)
-        self.__lists.get_model().append(row=[playlist])
-        self._select_playlist(playlist)
+        playlist = FileBackedPlaylist.new(PLAYLISTS)
+        self.model.append(row=[playlist])
+        self._select_playlist(playlist, scroll=True)
+
+        model, iter = self.__view.get_selection().get_selected()
+        path = model.get_path(iter)
+        GLib.idle_add(self._start_rename, path)
 
     def __start_editing(self, render, editable, path):
         editable.set_text(self.__lists[path][0].name)
 
     def __edited(self, render, path, newname):
+        return self._rename(path, newname)
+
+    def _rename(self, path, newname):
+        playlist = self.__lists[path][0]
         try:
-            self.__lists[path][0].rename(newname)
-        except ValueError, s:
+            playlist.rename(newname)
+        except ValueError as s:
             qltk.ErrorMessage(
                 None, _("Unable to rename playlist"), s).run()
         else:
             row = self.__lists[path]
-            self.__lists.row_changed(row.path, row.iter)
+            child_model = self.model
+            child_model.remove(
+                self.__lists.convert_iter_to_child_iter(row.iter))
+            child_model.append(row=[playlist])
+            self._select_playlist(playlist, scroll=True)
 
     def __import(self, activator, library):
         filt = lambda fn: fn.endswith(".pls") or fn.endswith(".m3u")
@@ -533,6 +588,9 @@ class PlaylistsBrowser(Browser):
 
     can_reorder = True
 
+    def scroll(self, song):
+        self.__view.iter_select_by_func(lambda r: song in r[0])
+
     def reordered(self, songs):
         model, iter = self.__view.get_selection().get_selected()
         playlist = None
@@ -540,7 +598,26 @@ class PlaylistsBrowser(Browser):
             playlist = model[iter][0]
             playlist[:] = songs
         elif songs:
-            playlist = Playlist.fromsongs(PLAYLISTS, songs)
+            playlist = FileBackedPlaylist.from_songs(PLAYLISTS, songs)
             GLib.idle_add(self._select_playlist, playlist)
         if playlist:
             PlaylistsBrowser.changed(playlist, refresh=False)
+
+
+class PreferencesButton(Gtk.HBox):
+    def __init__(self, browser):
+        super(PreferencesButton, self).__init__()
+
+        menu = Gtk.Menu()
+
+        pref_item = MenuItem(_("_Preferences"), Icons.PREFERENCES_SYSTEM)
+        menu.append(pref_item)
+        connect_obj(pref_item, "activate", Preferences, browser)
+
+        menu.show_all()
+
+        button = MenuButton(
+                SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.MENU),
+                arrow=True)
+        button.set_menu(menu)
+        self.pack_start(button, True, True, 0)
