@@ -6,7 +6,7 @@ from quodlibet.plugins.playorder import PlayOrderPlugin, PlayOrderInOrderMixin
 class ReverseOrder(PlayOrderPlugin, PlayOrderInOrderMixin):
     PLUGIN_ID = "reverse"
     PLUGIN_NAME = _("Reverse")
-    PLUGIN_ICON = Icons.VIEW_REFRESH
+    PLUGIN_ICON = Icons.MEDIA_SKIP_BACKWARD
     PLUGIN_DESC = _("Reverses the play order of songs.")
 
     def previous(self, playlist, iter):
