@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
+#           2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -7,6 +8,7 @@
 
 from quodlibet.qltk import Icons
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
+from quodlibet.plugins.songshelpers import any_song, is_writable
 
 
 class ForceWrite(SongsMenuPlugin):
@@ -16,5 +18,7 @@ class ForceWrite(SongsMenuPlugin):
                     "and ratings are up to date.")
     PLUGIN_ICON = Icons.DOCUMENT_SAVE
 
+    plugin_handles = any_song(is_writable)
+
     def plugin_song(self, song):
         song._needs_write = True
