@@ -10,8 +10,8 @@ import mutagen.id3
 
 from quodlibet import config, const, print_d
 from quodlibet import util
-from quodlibet.config import RATINGS
-from quodlibet.util.massagers import LanguageMassager
+from quodlibet.compat import iteritems
+from quodlibet.util.iso639 import ISO_639_2
 from quodlibet.util.path import get_temp_cover_file
 from quodlibet.util.string import isascii
 
@@ -80,7 +80,7 @@ class ID3File(AudioFile):
            # TLAN requires an ISO 639-2 language code, check manually
            #"TLAN": "language"
     }
-    SDI = dict([(v, k) for k, v in IDS.iteritems()])
+    SDI = dict([(v, k) for k, v in iteritems(IDS)])
 
     # At various times, information for this came from
     # http://musicbrainz.org/docs/specs/metadata_tags.html
@@ -101,7 +101,7 @@ class ID3File(AudioFile):
         u"ALBUMARTISTSORT": "albumartistsort",
         u"BARCODE": "barcode",
         }
-    PAM_XXXT = dict([(v, k) for k, v in TXXX_MAP.iteritems()])
+    PAM_XXXT = dict([(v, k) for k, v in iteritems(TXXX_MAP)])
 
     Kind = None
 
@@ -166,6 +166,9 @@ class ID3File(AudioFile):
             elif frame.FrameID == "TLAN":
                 self["language"] = "\n".join(frame.text)
                 continue
+            elif (frame.FrameID == "USLT" and frame.desc == "" and
+                  frame.lang == "\x00\x00\x00"):
+                name = "lyrics"
             else:
                 name = self.IDS.get(frame.FrameID, "").lower()
 
@@ -179,6 +182,9 @@ class ID3File(AudioFile):
                 text = "\n".join(map(unicode, frame.text))
             elif id3id == "COMM":
                 text = "\n".join(frame.text)
+            elif id3id == "USLT":
+                # lyrics are single string, not list
+                text = frame.text
             elif id3id.startswith("W"):
                 text = frame.url
                 frame.encoding = 0
@@ -302,7 +308,7 @@ class ID3File(AudioFile):
             else:
                 tag.add(Kind(encoding=enc, text=text))
 
-        dontwrite = ["genre", "comment", "musicbrainz_trackid"] \
+        dontwrite = ["genre", "comment", "musicbrainz_trackid", "lyrics"] \
             + RG_KEYS + self.TXXX_MAP.values()
 
         if "musicbrainz_trackid" in self.realkeys():
@@ -314,7 +320,7 @@ class ID3File(AudioFile):
         tag.delall("TLAN")
         if "language" in self:
             langs = self["language"].split("\n")
-            if all([lang in LanguageMassager.ISO_639_2 for lang in langs]):
+            if all([lang in ISO_639_2 for lang in langs]):
                 # Save value(s) to TLAN tag. Guaranteed to be ASCII here
                 tag.add(mutagen.id3.TLAN(encoding=3, text=langs))
                 dontwrite.append("language")
@@ -356,6 +362,13 @@ class ID3File(AudioFile):
             tag.add(mutagen.id3.COMM(encoding=enc, text=t, desc=u"",
                                      lang="\x00\x00\x00"))
 
+        tag.delall("USLT::\x00\x00\x00")
+        if "lyrics" in self:
+            enc = encoding_for(self["lyrics"])
+            # lyrics are single string, not array
+            tag.add(mutagen.id3.USLT(encoding=enc, text=self["lyrics"],
+                                     desc=u"", lang="\x00\x00\x00"))
+
         # Delete old foobar replaygain ..
         for frame in tag.getall("TXXX"):
             if frame.desc.lower() in RG_KEYS:
@@ -398,6 +411,9 @@ class ID3File(AudioFile):
             except KeyError:
                 pass
         for key in self.PAM_XXXT:
+            if key in self.SDI:
+                # we already write it back using non-TXXX frames
+                continue
             if key in self:
                 value = self[key]
                 f = mutagen.id3.TXXX(encoding=encoding_for(value),
@@ -406,8 +422,7 @@ class ID3File(AudioFile):
                 tag.add(f)
 
         if (config.getboolean("editing", "save_to_songs") and
-                (self("~#rating") != RATINGS.default or
-                 self.get("~#playcount", 0) != 0)):
+                (self.has_rating or self.get("~#playcount", 0) != 0)):
             email = config.get("editing", "save_email").strip()
             email = email or const.EMAIL
             t = mutagen.id3.POPM(email=email,
