@@ -15,7 +15,6 @@ from mutagen.id3 import ID3
 
 from quodlibet import config
 from quodlibet import const
-from quodlibet.config import RATINGS
 from quodlibet.util.path import get_temp_cover_file
 
 from ._audio import AudioFile
@@ -255,9 +254,8 @@ class MutagenVCFile(AudioFile):
 
         if config.getboolean("editing", "save_to_songs"):
             email = email or const.EMAIL
-            rating = self("~#rating")
-            if rating != RATINGS.default:
-                comments["rating:" + email] = str(rating)
+            if self.has_rating:
+                comments["rating:" + email] = str(self("~#rating"))
             playcount = self.get("~#playcount", 0)
             if playcount != 0:
                 comments["playcount:" + email] = str(playcount)
@@ -462,7 +460,7 @@ class FLACFile(MutagenVCFile):
         super(FLACFile, self).write()
 
 types = []
-for var in globals().values():
+for var in list(globals().values()):
     if getattr(var, 'MutagenType', None):
         types.append(var)
 
